@@ -2,7 +2,7 @@
  *
  * $Id: entry.h,v 1.36 2001/03/19 19:27:40 root Exp $
  *
- * Copyright (C) 1997-2003 by Dimitri van Heesch.
+ * Copyright (C) 1997-2005 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -38,7 +38,8 @@ struct ListItemInfo
 struct BaseInfo 
 {
   /*! Creates an object representing an inheritance relation */
-  BaseInfo(const char *n,Protection p,Specifier v) : name(n),prot(p),virt(v) {}
+  BaseInfo(const char *n,Protection p,Specifier v) : 
+    name(n),prot(p),virt(v) {}
   QCString   name; //!< the name of the base class
   Protection prot; //!< inheritance type
   Specifier  virt; //!< virtualness
@@ -135,9 +136,9 @@ struct Grouping
   {
     GROUPING_LOWEST,
     GROUPING_AUTO_WEAK = 
-      GROUPING_LOWEST,   //!< membership in group was defined via \@weakgroup 
-    GROUPING_AUTO_ADD,    //!< membership in group was defined via \@add[to]group 
-    GROUPING_AUTO_DEF,    //!< membership in group was defined via \@defgroup
+      GROUPING_LOWEST,     //!< membership in group was defined via \@weakgroup 
+    GROUPING_AUTO_ADD,     //!< membership in group was defined via \@add[to]group 
+    GROUPING_AUTO_DEF,     //!< membership in group was defined via \@defgroup
     GROUPING_AUTO_HIGHEST = GROUPING_AUTO_DEF,
     GROUPING_INGROUP,      //!< membership in group was defined by \@ingroup
     GROUPING_HIGHEST = GROUPING_INGROUP
@@ -184,53 +185,65 @@ class Entry
       EXCEPTION_SEC    = 0x00000008,
       NAMESPACE_SEC    = 0x00000010,
       INTERFACE_SEC    = 0x00000020,
+      PROTOCOL_SEC     = 0x00000040,
+      CATEGORY_SEC     = 0x00000080,
       COMPOUND_MASK    = CLASS_SEC | STRUCT_SEC | UNION_SEC | 
-                         INTERFACE_SEC | EXCEPTION_SEC,
+                         INTERFACE_SEC | EXCEPTION_SEC | 
+                         PROTOCOL_SEC | CATEGORY_SEC,
       SCOPE_MASK       = COMPOUND_MASK | NAMESPACE_SEC,
       
-      CLASSDOC_SEC     = 0x00000100, 
-      STRUCTDOC_SEC    = 0x00000200,
-      UNIONDOC_SEC     = 0x00000400,
-      EXCEPTIONDOC_SEC = 0x00000800,
-      NAMESPACEDOC_SEC = 0x00001000,
-      INTERFACEDOC_SEC = 0x00002000,
+      CLASSDOC_SEC     = 0x00000800, 
+      STRUCTDOC_SEC    = 0x00001000,
+      UNIONDOC_SEC     = 0x00002000,
+      EXCEPTIONDOC_SEC = 0x00004000,
+      NAMESPACEDOC_SEC = 0x00008000,
+      INTERFACEDOC_SEC = 0x00010000,
+      PROTOCOLDOC_SEC  = 0x00020000,
+      CATEGORYDOC_SEC  = 0x00040000,
       COMPOUNDDOC_MASK = CLASSDOC_SEC | STRUCTDOC_SEC | UNIONDOC_SEC | 
-                         INTERFACEDOC_SEC | EXCEPTIONDOC_SEC,
+                         INTERFACEDOC_SEC | EXCEPTIONDOC_SEC | PROTOCOLDOC_SEC |
+                         CATEGORYDOC_SEC,
 
-      SOURCE_SEC       = 0x00010000,
-      HEADER_SEC       = 0x00020000,
+      SOURCE_SEC       = 0x00400000,
+      HEADER_SEC       = 0x00800000,
       FILE_MASK        = SOURCE_SEC | HEADER_SEC,
 
-      ENUMDOC_SEC      = 0x00100000,
-      ENUM_SEC         = 0x00200000,
-      EMPTY_SEC        = 0x00300000, 
-      PAGEDOC_SEC      = 0x00400000, 
-      VARIABLE_SEC     = 0x00500000,
-      FUNCTION_SEC     = 0x00600000,
-      TYPEDEF_SEC      = 0x00700000,
-      MEMBERDOC_SEC    = 0x00800000, 
-      OVERLOADDOC_SEC  = 0x00900000,
-      EXAMPLE_SEC      = 0x00a00000, 
-      VARIABLEDOC_SEC  = 0x00b00000,
-      FILEDOC_SEC      = 0x00c00000,
-      DEFINEDOC_SEC    = 0x00d00000,
-      INCLUDE_SEC      = 0x00e00000,
-      DEFINE_SEC       = 0x00f00000,
-      GROUPDOC_SEC     = 0x01000000,
-      USINGDIR_SEC     = 0x01100000,
-      MAINPAGEDOC_SEC  = 0x01200000,
-      MEMBERGRP_SEC    = 0x01300000,
-      USINGDECL_SEC    = 0x01400000,
-      PACKAGE_SEC      = 0x01500000,
-      PACKAGEDOC_SEC   = 0x01600000
+      ENUMDOC_SEC      = 0x01000000,
+      ENUM_SEC         = 0x02000000,
+      EMPTY_SEC        = 0x03000000, 
+      PAGEDOC_SEC      = 0x04000000, 
+      VARIABLE_SEC     = 0x05000000,
+      FUNCTION_SEC     = 0x06000000,
+      TYPEDEF_SEC      = 0x07000000,
+      MEMBERDOC_SEC    = 0x08000000, 
+      OVERLOADDOC_SEC  = 0x09000000,
+      EXAMPLE_SEC      = 0x0a000000, 
+      VARIABLEDOC_SEC  = 0x0b000000,
+      FILEDOC_SEC      = 0x0c000000,
+      DEFINEDOC_SEC    = 0x0d000000,
+      INCLUDE_SEC      = 0x0e000000,
+      DEFINE_SEC       = 0x0f000000,
+      GROUPDOC_SEC     = 0x10000000,
+      USINGDIR_SEC     = 0x11000000,
+      MAINPAGEDOC_SEC  = 0x12000000,
+      MEMBERGRP_SEC    = 0x13000000,
+      USINGDECL_SEC    = 0x14000000,
+      PACKAGE_SEC      = 0x15000000,
+      PACKAGEDOC_SEC   = 0x16000000,
+      OBJCIMPL_SEC     = 0x17000000,
+      DIRDOC_SEC       = 0x18000000,
     };
     enum MemberSpecifier
     {
-      Inline    = 0x01,
-      Explicit  = 0x02,
-      Mutable   = 0x04,
-      Settable  = 0x08,
-      Gettable  = 0x10
+      Inline    = 0x0001,
+      Explicit  = 0x0002,
+      Mutable   = 0x0004,
+      Settable  = 0x0008,
+      Gettable  = 0x0010,
+      Readable  = 0x0020,
+      Writable  = 0x0040,
+      Final     = 0x0080,
+      Abstract  = 0x0100,
     };
 
     Entry();
@@ -274,8 +287,13 @@ class Entry
     QCString     brief;       //!< brief description (doc block)
     int          briefLine;   //!< line number at which the brief desc. was found
     QCString     briefFile;   //!< file in which the brief desc. was found
+    QCString     inbodyDocs;  //!< documentation inside the body of a function
+    int          inbodyLine;  //!< line number at which the body doc was found
+    QCString     inbodyFile;  //!< file in which the body doc was found
     QCString     relates;     //!< related class (doc block)
     bool         relatesDup;  //!< keep duplicate doc in original file also
+    QCString     read;        //!< property read accessor
+    QCString     write;       //!< property write accessor
     QCString     inside;      //!< name of the class in which documents are found
     QCString     exception;   //!< throw specification
     int          bodyLine;    //!< line number of the definition in the source
@@ -290,11 +308,12 @@ class Entry
     QList<ListItemInfo> *sli; //!< special lists (test/todo/bug/deprecated/..) this entry is in
     TagInfo    *tagInfo;      //!< tag file info
     static int  num;          //!< counts the total number of entries
+    bool        objc;         //!< Objective-C construct
     enum 
     {
-      GROUPDOC_NORMAL,        //<! defgroup
-      GROUPDOC_ADD,           //<! addgroup
-      GROUPDOC_WEAK           //<! weakgroup
+      GROUPDOC_NORMAL,        //!< defgroup
+      GROUPDOC_ADD,           //!< addgroup
+      GROUPDOC_WEAK           //!< weakgroup
     } groupDocType;           //!< kind of group
     /// return the command name used to define GROUPDOC_SEC
     const char *groupDocCmd() const
